
/*********** PUBLIC FUNCTIONS ***********
    
    onLoad           ()
    showInterstitial ()
    exitApp          ()
    
************ LOCAL FUNCTIONS ************

    onDeviceReady       ()
    setDevicePlatform   ()

    checkConnection     ()
    onConfirmConnection (buttonIndex)
    
    initAd              ()
    registerAdEvents    ()
    removeBanner        ()
    
    onConfirmExit       (buttonIndex)

*****************************************/

(function($) {

    // --------- GLOBAL CONSTANTS ---------

    window.PLATFORM = "web";

    // --------- LOCAL CONSTANTS ----------

    var EXIT_MESSAGE         = "Sei sicuro di voler chiudere l'applicazione?",
        EXIT_BOX_TITLE       = "Uscita!",
        EXIT_POS_BUTTON_TEXT = "Sì",
        EXIT_NEG_BUTTON_TEXT = "No";

    var DEBUG = false;

    var adUnits = {
        "ios" : {
            "banner"       : "",
            "interstitial" : ""
        },
        "android" : {
            "banner"       : "ca-app-pub-3273263168540896/6282155768",
            "interstitial" : "ca-app-pub-3273263168540896/4805422568"
        }
    };
    var admobId = ( /(android)/i.test(navigator.userAgent) ) ? adUnits.android : adUnits.ios;

    // ------------------------------------

    // -----
    // -- PUBLIC FUNCTIONS
    // -----

    // Wait for device API libraries to load
    function onLoad() {
        document.addEventListener("deviceready", onDeviceReady, false);
    }

    // ------------------------

    function prepareInterstitial() {

        function onSuccess() {
            if (DEBUG) showAlertMessage("Interstitial prepared!");
        }

        function onError() {
            if (DEBUG) showAlertMessage("Error on prepare interstitial!");
        }

        AdMob.prepareInterstitial({
            "adId"      : admobId.interstitial,
            "autoShow"  : false,
            "isTesting" : DEBUG,
            "success"   : onSuccess,
            "error"     : onError
        });
    }

    function showInterstitial() {

        if (AdMob) {

            AdMob.isInterstitialReady(function(isReady) {

                if (isReady) {

                    AdMob.showInterstitial();

                    setTimeout(prepareInterstitial, 1000);

                } else {
                    if (DEBUG) showAlertMessage("Interstitial not ready!");
                }
            });
        }
    }

    // ------------------------

    function exitApp() {

        if (PLATFORM == "android") {

            if (navigator.notification) {
                navigator.notification.confirm(
                    EXIT_MESSAGE,    // Message
                    onConfirmExit,   // Callback
                    EXIT_BOX_TITLE,  // Title
                    [ EXIT_POS_BUTTON_TEXT, EXIT_NEG_BUTTON_TEXT ] // Buttons
                );

            } else if ( confirm(EXIT_MESSAGE) ) {
                onConfirmExit(1);            
            }
        }
    }

    // ------------------------

    window.onLoad           = onLoad;
    window.showInterstitial = showInterstitial;
    window.exitApp          = exitApp;

    // ------------------------------------

    // -----
    // -- LOCAL FUNCTIONS
    // -----

    // Device APIs are available
    function onDeviceReady() {

        // Set global constant PLATFORM
        setDevicePlatform();

        // iOS status bar problems
        if (window.PLATFORM == "ios" && typeof StatusBar !== 'undefined') {
            StatusBar.hide();
        }

        // Disable exit button if not Android
        if (PLATFORM == "android") {
            $("#settings-menu .settings-item").last().show();
        }

        // Fix window orientation
        window.screen.orientation.lock("portrait");

        // Set back button event
        document.addEventListener("backbutton", exitApp, false);

        // Set menu button event

        if (typeof navigator.app !== 'undefined') { // app not supported by iOS
            navigator.app.overrideButton("menubutton", true);  // Without this line the 'menubutton' event is not fired
        }
        document.addEventListener("menubutton", toggleSettings, false);

        // Check connection
        checkConnection();
    }

    function setDevicePlatform() {

        var platformLower = device.platform.toLowerCase();

        if ( platformLower.indexOf("android") >= 0 ) {
            window.PLATFORM = "android";

        } else if ( platformLower.indexOf("ios") >= 0 ) {
            window.PLATFORM = "ios";

        } else if ( platformLower.indexOf("browser") >= 0 ) {
            window.PLATFORM = "browser";

        } else if ( platformLower.indexOf("blackberry") >= 0 ) {
            window.PLATFORM = "blackberry";
    
        } else if ( platformLower.indexOf("win") >= 0 ) {
            window.PLATFORM = "windows";
    
        } else if ( platformLower.indexOf("tizen") >= 0 ) {
            window.PLATFORM = "tizen";
    
        } else if ( platformLower.indexOf("mac") >= 0 ) {
            window.PLATFORM = "mac";

        } else {
            window.PLATFORM = "unknown";
        }
    }

    // ------------------------

    function checkConnection() {

        if (navigator.connection.type == Connection.NONE) {

            navigator.notification.confirm(
                "Attenzione! Connessione a internet necessaria per questa applicazione!\nVuoi riprovare?",  // Message
                onConfirmConnection,    // Callback
                "Connessione assente",  // Title
                ["Riprova", "Chiudi"]   // Buttons
            );

        } else {
            // Init Ad Mob banners
            initAd();
        }
    }

    function onConfirmConnection(buttonIndex) {

        if (buttonIndex == 1) {
            checkConnection();

        } else if (PLATFORM == "android") {
            navigator.app.exitApp();
        }            
    }
    
    // ------------------------

    function initAd() {

        if (AdMob) {

            registerAdEvents();

            // showInterstitial();

            // Show AdMob Banner
            setTimeout(function() {

                AdMob.createBanner({
                    "adId"      : admobId.banner,
                    "adSize"    : AdMob.AD_SIZE.SMART_BANNER,
                    "position"  : AdMob.AD_POSITION.TOP_CENTER,
                    "isTesting" : DEBUG,
                    "overlap"   : false, // set to true, to allow banner overlap webview
                    "autoShow"  : true
                });

                prepareInterstitial();

            }, 2000);

        } else {
            showAlertMessage("AdMob plugin not ready");
        }
    }

    function registerAdEvents() {

        // document.addEventListener('onReceiveAd', function(){});
        // document.addEventListener('onFailedToReceiveAd', function(data){});
        // document.addEventListener('onPresentAd', function(){});
        // document.addEventListener('onDismissAd', function(){ });
        // document.addEventListener('onLeaveToAd', function(){ });
        // document.addEventListener('onReceiveInterstitialAd', function(){ });
        // document.addEventListener('onPresentInterstitialAd', function(){ });
        // document.addEventListener('onDismissInterstitialAd', function(){
        //     AdMob.createInterstitialView(); // REMOVE THESE 2 LINES IF USING AUTOSHOW 
        //     AdMob.requestInterstitialAd();  // get the next one ready only after the current one is closed 
        // });
    }

    function removeBanner() {

        if (AdMob) {
            AdMob.removeBanner();
        }
    }

    // ------------------------

    function onConfirmExit(buttonIndex) {

        if (buttonIndex == 1) {

            removeBanner();

            if (navigator.app) {
                navigator.app.exitApp();
            
            } else {
                toggleSettings();
            }
        }
    }

    // ------------------------

    function showAlertMessage(message) {

        if (navigator.notification && navigator.notification.alert) {
            navigator.notification.alert(message);
        } else {
            alert(message);                
        }
    }

})(jQuery);
