
/*********** PUBLIC FUNCTIONS ***********

    analyze        ()

    toggleSettings ()

************ LOCAL FUNCTIONS ************
    
    setDOMevents           ()

    showWelcomePopup       ()
    clickOnWelcomeButton   ()

    setLogicalAnalysis     (data)
    setGrammaticalAnalysis (data)

    showOldVersionPopup    ()
    clickOnUpdateButton    ()

    sendErrorCheck         ()
    onConfirmError         (buttonIndex)
    sendError              ()
    showSentBox            ()
    
    openSettings           (bypassCheck)
    closeSettings          (bypassCheck)
    clickOnSettingsLayer   ()

    openCredits            ()
    closeCredits           ()

    openPanel              ()

    expandLogicalItem      ()

    clickOnSettings        ()
    clickOnClearInput      ()

    contentPanStart        (e)
    contentPanEnd          (e)

    slideToLogic           ()
    slideToGrammar         ()

    windowResize           ()

*****************************************/

(function($) {

    "use strict";

    // --------- LOCAL CONSTANTS ----------

    var DEBUG                           = false,
        BACKEND_URL                     = "http://eieiteam.com/services/lg_analysis/backend.php",
        PLAY_STORE_URL                  = "market://details?id=com.eiei.lganalysis",
        APP_VERSION                     = "1.0.5",
        SHOW_INTERSTITIAL_REQUEST_COUNT = 5,
        SWIPE_X_THRESHOLD               = 30,  // in pixel
        SWIPE_Y_THRESHOLD               = 75,  // in pixel
        ANIMATION_TIME_PANEL            = 200,
        ANIMATION_TIME_COLLAPSIBLE      = 300,
        ANIMATION_TIME_CONTENTS         = 100,  // for each content
        ANIMATION_TIME_SWIPE            = 300,
        INTERSTITIAL_LOADING_TIMEOUT    = 3000,
        SENT_BOX_HIDING_TIME            = 1500;

    // Texts
    var DEFAULT_INPUT_TEXT  = "Questa è una frase di prova.",
        AJAX_ERROR_MESSAGE  = "Errore! Impossibile connettersi al servizio, riprova più tardi.",
        LABEL_NOT_AVAILABLE = "";

    // ----------- LOCAL FLAGS ------------

    var creditsOpen = false,
        isSliding   = false;

    // ------------ LOCAL VARS ------------

    var containerHeight = null,
        textInput       = null,
        textAnalysis    = null,
        mouseX          = 0,
        mouseY          = 0,
        requestCount    = 0;

    // ------------------------------------

    // -----
    // -- DOM EVENTS
    // -----
    
    // Logical item click
    $(document).on("click", "#content-logical .logical-phrase-item", expandLogicalItem);

    // Resize
    $(window).resize(windowResize);

    // Ready
    $(document).on("pageinit ready", function() {

        // Set DOM events
        setDOMevents();

        // Set input text placeholder
        $("#text-input > input").attr("placeholder", DEFAULT_INPUT_TEXT);

        // Welcome popup
        showWelcomePopup();

        // Initial resize
        windowResize();
    });

    //-------------------------------------

    // -----
    // -- PUBLIC FUNCTIONS
    // -----

    String.prototype.capitalize = function() {
        return this.charAt(0).toUpperCase() + this.slice(1);
    };

    // ------------------------

    function analyze() {

        function ajaxError(xhr, textStatus, errorThrown) {

            if (DEBUG) {
                alert(JSON.stringify(xhr));

            } else {
                alert(AJAX_ERROR_MESSAGE);
            }

            $("#loading-img").hide();
            $("#welcome-text").show();
        }

        function ajaxSuccess(data) {

            if (!data.last_version) {
                // Show popup for old versions
                showOldVersionPopup();

            } else {

                // Save analysis
                textAnalysis = data.analysis;

                // Remove connection text
                $("#connection-text").remove();
                
                // Show panel
                $("#panel").show();
                openPanel();
                
                // Show content
                $("#content").show();

                // Show interstitial
                if (requestCount++ % SHOW_INTERSTITIAL_REQUEST_COUNT == 0) {

                    showInterstitial();

                    // Show results after a pre-defined time
                    setTimeout(function() {
                        showResults(textAnalysis);
                    }, INTERSTITIAL_LOADING_TIMEOUT);

                } else {
                    showResults(textAnalysis);
                }

                // Enable settings items (Send error)
                $("#settings-menu .settings-item.button-disabled").removeClass("button-disabled");

                // Resize
                windowResize();
            }
        }

        // Set global var
        textInput = $("#text-input input").val();

        // Set default input text
        if (textInput == "") {
            textInput = DEFAULT_INPUT_TEXT;
            if (DEBUG) textInput = "Marco mangia la mela, mentre Matteo, che si sente un po' grasso, mangia la banana che è piena di potassio."
        }
        
        // Close keyboard
        $("input").blur();

        // Hide welcome text
        $("#welcome-text").hide();

        // Hide sections
        $("#content-logical").hide();
        $("#content-grammatical").hide();
        
        // Show loading
        $("#loading-img").show();
        
        // Backend call
        $.ajax({
            "url"         : BACKEND_URL,
            "async"       : true,
            "type"        : "POST",
            "dataType"    : "json",
            "contentType" : "application/x-www-form-urlencoded; charset=UTF-8",
            "data"        : {
                "cmd"     : "analyze",
                "text"    : textInput,
                "version" : APP_VERSION
            },
            "success"     : ajaxSuccess,
            "error"       : ajaxError
        });
    }

    // ------------------------

    function toggleSettings() {

        if ( $("#settings-layer").hasClass("layer-visible") ) {
            closeSettings(true);
        } else {
            openSettings(true);
        }
    }

    // ------------------------

    window.analyze        = analyze;
    window.toggleSettings = toggleSettings;

    // ------------------------------------

    // -----
    // -- LOCAL FUNCTIONS
    // -----

    function setDOMevents() {
    
        // Welcome button click
        $("#welcome-button").click(clickOnWelcomeButton);

        // Update button click
        $("#update-button").click(clickOnUpdateButton);

        // Settings button click
        $("#settings-button").click(toggleSettings);

        // Settings layer click
        $("#settings-layer").click(clickOnSettingsLayer);
            
        // Credits close button click
        $("#credits-button").click(closeCredits);
        
        // Clear input click
        $("#clear-input").click(clickOnClearInput);
    
        // Settings menu click
        $("#settings-menu").click(function(e) {
            e.stopPropagation();
        });

        // Settings click
        $("#settings-menu .settings-item").click(clickOnSettings);

        // Allows :active style to be triggered on mobile device
        $(".hover-element").on("touchstart", function() {
            return true;
        });

        // Close keyboard on touch
        $("#top-container, #text-form, #content").on("vmousedown", function() {
            $("input").blur();  // keyboard closed
        });
        
        // Event to show/hide the 'X' to clear input 
        $("#text-input > input").keyup(function() {
            if ($(this).val() == "") {
                $("#clear-input").hide();
            } else {
                $("#clear-input").show();
            }
        });

        // Set pan events
        var hammerHandler = new Hammer(document.getElementById("content"));
        hammerHandler.on("panstart", contentPanStart);
        hammerHandler.on("panend", contentPanEnd);

        // Set swipe events
        $("#content").on("swiperight", slideToLogic);
        $("#content").on("swipeleft", slideToGrammar);

        // Panel buttons click
        $("#button-logical").click(slideToLogic);
        $("#button-grammatical").click(slideToGrammar);
    }

    // ------------------------

    function showWelcomePopup() {

        var storage   = window.localStorage,
            firstTime = storage.getItem("firstTime");

        if (firstTime != "false") {
            $("#welcome-popup").fadeIn();
        } 
    }

    function clickOnWelcomeButton() {

        var storage = window.localStorage;

        storage.setItem("firstTime", "false");
        $("#welcome-popup").stop(true).fadeOut();
    }
    
    // ------------------------

    function showResults(textAnalysis) {

        setLogicalAnalysis(textAnalysis.logical);
        setGrammaticalAnalysis(textAnalysis.grammatical);

        $("#loading-img").hide();
    }

    function setLogicalAnalysis(data) {

        // Delete logical analysis content
        $("#content-logical").hide().html("");

        for (var i in data) {
            var phrase        = data[i],
                DOMphraseItem = $('<div class="logical-phrase-item hover-element">\
                                    <div class="parsed-phrase">' + phrase.text + '</div>\
                                    <div class="phrase-label">' + phrase.label + '</div>\
                                    <div class="dots"><span>. . .</span></div>\
                                </div>'),
                itemList = $('<div class="logical-item-analysis-list"></div>');

            for (var j in phrase.analysis) {
                var listElement = phrase.analysis[j];

                if (!listElement.punct) {

                    var label   = listElement.label != '' ? listElement.label : LABEL_NOT_AVAILABLE,
                        DOMitem = $('<div class="logical-analysis-item">\
                                        <div class="parsed-logical-form">' + listElement.form + '</div>\
                                        <div class="parsed-logical-label">' + label.capitalize() + '</div>\
                                    </div>');

                    if (listElement.main_verb) {
                        DOMitem.addClass("main-verb");
                    }

                    itemList.append(DOMitem);
                }
            }

            // Append to item
            DOMphraseItem.append(itemList);

            // Append to container
            $("#content-logical").append(DOMphraseItem);
        }

        // Append bottom padding
        $("#content-logical").append('<div class="bottom-padding"></div>');

        // Auto expand if there is only a sentence
        if (data.length == 1) {
            $("#content-logical .logical-phrase-item").click();
        }

        // Show logical content sliding
        if ($("#button-logical").hasClass("button-focused")) {
            $("#content-logical").slideDown( ANIMATION_TIME_CONTENTS * data.length );
        }
    }

    function setGrammaticalAnalysis(data) {

        // Delete grammatical analysis content
        $("#content-grammatical").hide().html("");

        for (var i in data) { 
            var token = data[i];

            if (token.pos != "PUNCT") {

                var DOMitem  = $('<div class="grammatical-analysis-item">\
                                    <div class="parsed-grammatical-form">' + token.form + '</div>\
                                    <div class="parsed-grammatical-label">' + token.label.capitalize() + '</div>\
                                    <div class="parsed-grammatical-info-list"></div>\
                                </div>'),
                    itemInfo = "";

                for (var j in token.info) {
                    var info = token.info[j];

                    itemInfo += '<div class="parsed-grammatical-info">\
                                    <div class="grammatical-info-label border-box">' + info.label + '</div>\
                                    <div class="grammatical-info-value border-box">' + info.value + '</div>\
                                </div>';
                }

                // Append info to item
                DOMitem.find(".parsed-grammatical-info-list").append(itemInfo);

                // Append item to container
                $("#content-grammatical").append(DOMitem);
            }
        }

        // Append bottom padding
        $("#content-grammatical").append('<div class="bottom-padding"></div>');

        // Show grammatical content sliding
        if ($("#button-grammatical").hasClass("button-focused")) {
            $("#content-grammatical").slideDown(ANIMATION_TIME_CONTENTS * data.length); // n_phrases ???
        }
    }

    // ------------------------

    function showOldVersionPopup() {
        $("#old-versions-popup").fadeIn();
    }
    
    function clickOnUpdateButton() {
        window.open(PLAY_STORE_URL, '_blank');
    }

    // ------------------------

    function sendErrorCheck() {

        if (navigator.notification) {
            navigator.notification.confirm(
                "L'analisi effettuata dall'applicazione risulta errata?",  // Message
                onConfirmError,       // Callback
                "Errore di analisi",  // Title
                ["Sì", "No"]          // Buttons
            );

        } else if ( confirm("L'analisi effettuata dall'applicazione risulta errata?") ) {
            sendError();            
        }
    }

    function onConfirmError(buttonIndex) {

        if (buttonIndex == 1) {
            sendError();
        }
    }

    function sendError() {

        function ajaxError(xhr, textStatus, errorThrown) {

            var alertMessage = DEBUG ? JSON.stringify(xhr) : AJAX_ERROR_MESSAGE;

            if (navigator.notification) {
                navigator.notification.alert( alertMessage );
            } else {
                alert( alertMessage );
            }
        }

        function ajaxSuccess() {

            // Close settings
            closeSettings();
            
            // Show sent box
            showSentBox();
        }

        // ------------

        if (textInput != null) {

            $.ajax({
                "url"         : BACKEND_URL,
                "async"       : true,
                "type"        : "POST",
                "contentType" : "application/x-www-form-urlencoded; charset=UTF-8",
                "data"        : {
                    "cmd"      : "send_error",
                    "text"     : textInput,
                    "analysis" : textAnalysis
                },
                "success"     : ajaxSuccess,
                "error"       : ajaxError
            });
        }
    }

    function showSentBox() {

        if ( $("#sent-box").children().length == 0 ) {
            // Create sent box
            $("#sent-box").freeow("Segnalazione inviata!", "", {
                "classes"       : ["gray", "pushpin"],
                "autoHide"      : true,
                "autoHideDelay" : SENT_BOX_HIDING_TIME
            });
        } else {
            // Show sent box
            $("#sent-box").freeow();
        }

        // Show sent box container
        $("#sent-box").show();

        // Hide sent box container
        setTimeout(function() {
            $("#sent-box").hide();
        }, SENT_BOX_HIDING_TIME);
    }

    // ------------------------

    function openSettings(bypassCheck) {

        if ( bypassCheck || !$("#settings-layer").hasClass("layer-visible") ) {
            $("#settings-layer").addClass("layer-visible");
            $("#settings-layer").stop(true).fadeIn();
        }
    }

    function closeSettings(bypassCheck) {

        if ( bypassCheck || $("#settings-layer").hasClass("layer-visible") ) {
            $("#settings-layer").removeClass("layer-visible");
            $("#settings-layer").stop(true).fadeOut();
        }
    }

    function clickOnSettingsLayer() {
        
        if (!creditsOpen) {
            closeSettings();
        }
    }

    // ------------------------

    function openCredits() {

        if (!creditsOpen) {
            creditsOpen = true;
            $("#credits").fadeIn();
        }
    }

    function closeCredits() {

        if (creditsOpen) {

            $("#credits").fadeOut(function() {
                creditsOpen = false;
            });
        }
    }

    // ------------------------

    function openPanel() {

        // Animation
        $("#panel").animate({"bottom" : 0}, ANIMATION_TIME_PANEL);
    }

    // ------------------------

    function expandLogicalItem() {

        var analysisList = $(this).find(".logical-item-analysis-list"),
            dots         = $(this).find(".dots > span");

        if (analysisList.hasClass("list-expanded")) {
            // Collapse
            analysisList.removeClass("list-expanded");
            dots.stop(true).slideDown(ANIMATION_TIME_COLLAPSIBLE);
            analysisList.stop(true).slideUp(ANIMATION_TIME_COLLAPSIBLE);

        } else {
            // Expand
            analysisList.addClass("list-expanded");
            dots.stop(true).slideUp(ANIMATION_TIME_COLLAPSIBLE);
            analysisList.stop(true).slideDown(ANIMATION_TIME_COLLAPSIBLE);
        }
    }

    // ------------------------

    function clickOnSettings() {

        if ( !$(this).hasClass("button-disabled") ) {

            var buttonIndex = parseInt($(this).attr("itemprop"));

            switch (buttonIndex) {

                case 0:
                    closeSettings();
                    break;

                case 1:
                    sendErrorCheck();
                    break;

                case 2:
                    openCredits();
                    break;

                case 3:
                    exitApp();
                    break;

                default:
                    throw "Invalid button index: " + buttonIndex;
            }
        }
    }

    function clickOnClearInput() {
        $("#text-input > input").val("");
        $(this).hide();
    }

    // ------------------------

    function contentPanStart(e) {

        mouseX = e.center.x;
        mouseY = e.center.y;
    }

    function contentPanEnd(e) {

        var xDiff = e.center.x - mouseX,
            yDiff = e.center.y - mouseY;

        if (yDiff <= SWIPE_Y_THRESHOLD) {

            if ( xDiff >= SWIPE_X_THRESHOLD ) {
                $("#content").trigger("swiperight");
            
            } else if ( xDiff <= -SWIPE_X_THRESHOLD ) {
                $("#content").trigger("swipeleft");
            }
        }
    }

    // ------------------------

    function slideToLogic() {

        if ( !isSliding && !$("#content-logical").hasClass("content-visible") ) {

            isSliding = true;

            // Disable scrolling
            $("#content .page-content").addClass("scrolling-disabled");

            // Change button focus
            $("#panel .button-focused").removeClass("button-focused");
            $("#button-logical").addClass("button-focused");

            // Start sliding

            $("#content-grammatical").hide(
                "slide",
                {
                    "direction" : "right",
                    "easing"    : "linear"
                },
                ANIMATION_TIME_SWIPE);

            $("#content-logical").show(
                "slide",
                {
                    "direction" : "left",
                    "easing"    : "linear"
                },
                ANIMATION_TIME_SWIPE,
                function() {
                    // End sliding

                    // Set section visible
                    $(".content-visible").removeClass("content-visible");
                    $("#content-logical").addClass("content-visible");

                    // Enable scrolling
                    $("#content .page-content").removeClass("scrolling-disabled");

                    isSliding = false;
                }); 
        }
    }

    function slideToGrammar() {

        if ( !isSliding && !$("#content-grammatical").hasClass("content-visible") ) {

            isSliding = true;

            // Disable scrolling
            $("#content .page-content").addClass("scrolling-disabled");

            // Change button focus
            $("#panel .button-focused").removeClass("button-focused");
            $("#button-grammatical").addClass("button-focused");

            // Start sliding

            $("#content-logical").hide(
                "slide",
                {
                    "direction" : "left",
                    "easing"    : "linear"
                },
                ANIMATION_TIME_SWIPE);

            $("#content-grammatical").show(
                "slide",
                {
                    "direction" : "right",
                    "easing"    : "linear"
                },
                ANIMATION_TIME_SWIPE,
                function() {                            
                    // End sliding
                    
                    // Set section visible
                    $(".content-visible").removeClass("content-visible");
                    $("#content-grammatical").addClass("content-visible");

                    // Enable scrolling
                    $("#content .page-content").removeClass("scrolling-disabled");                         
                    
                    isSliding = false;
                });
        }
    }

    // ------------------------

    function windowResize() {

        var body            = $("body"),
            bodyWidth       = body.width(),
            bodyHeight      = body.height(),
            container       = $("#top-container"),
            connectionText  = $("#connection-text"),
            formHeight      = $("#text-form").outerHeight(true),
            content         = $("#content"),
            contentMargin   = parseInt( content.css("margin-top") ),
            contentHeight   = null;

        if (containerHeight == null) {
            containerHeight = container.height();
        }

        container.height(containerHeight + "px");

        if (bodyWidth > bodyHeight) {
            connectionText.hide();
        } else {
            connectionText.show();
        }

        contentHeight = containerHeight - formHeight - contentMargin;
        content.outerHeight(contentHeight + "px" );
    }

})(jQuery);
